package frc.robot.subsystems;

import com.kauailabs.navx.frc.AHRS;

import edu.wpi.first.math.geometry.Rotation2d;
import edu.wpi.first.math.kinematics.ChassisSpeeds;
import edu.wpi.first.math.kinematics.SwerveDriveKinematics;
import edu.wpi.first.math.kinematics.SwerveModuleState;
import edu.wpi.first.wpilibj.SPI.Port;
import edu.wpi.first.wpilibj2.command.SubsystemBase;
import frc.robot.Constants;
import frc.robot.SwervePod;

public class Chassis extends SubsystemBase {
  public enum Pod {
    LEFT_FRONT,
    RIGHT_FRONT,
    LEFT_BACK,
    RIGHT_BACK
  }

  private SwerveDriveKinematics m_kinematics;

  private SwervePod m_lf_pod = new SwervePod(Constants.LF_TURN, Constants.LF_DRIVE);
  private SwervePod m_rf_pod = new SwervePod(Constants.RF_TURN, Constants.RF_DRIVE);
  private SwervePod m_lb_pod = new SwervePod(Constants.LB_TURN, Constants.LB_DRIVE);
  private SwervePod m_rb_pod = new SwervePod(Constants.RB_TURN, Constants.RB_DRIVE);

  public AHRS m_gyro = new AHRS(Port.kMXP);

  public Chassis() {
    m_kinematics = new SwerveDriveKinematics(
      Constants.LF_TRANSLATION,
      Constants.RF_TRANSLATION,
      Constants.LB_TRANSLATION,
      Constants.RB_TRANSLATION);

    zeroPodEncoders();
    // zeroGyro();
  }

  public void drive(ChassisSpeeds speeds) {
    SwerveModuleState[] moduleStates = m_kinematics.toSwerveModuleStates(speeds);

    // Optimize pod angles so we arnt doing stupid wrapping
    SwerveModuleState lf_optimized = SwerveModuleState.optimize(
      moduleStates[0],
      Rotation2d.fromDegrees(getPod(Pod.LEFT_FRONT).getAngle())
    );
    SwerveModuleState rf_optimized = SwerveModuleState.optimize(
      moduleStates[1],
      Rotation2d.fromDegrees(getPod(Pod.RIGHT_FRONT).getAngle())
    );
    SwerveModuleState lb_optimized = SwerveModuleState.optimize(
      moduleStates[2],
      Rotation2d.fromDegrees(getPod(Pod.LEFT_BACK).getAngle())
    );
    SwerveModuleState rb_optimized = SwerveModuleState.optimize(
      moduleStates[3],
      Rotation2d.fromDegrees(getPod(Pod.RIGHT_BACK).getAngle())
    );

    // Set the pod's angle and velocity
    getPod(Pod.LEFT_FRONT).set(
      lf_optimized.angle.getDegrees(),
      lf_optimized.speedMetersPerSecond);
    getPod(Pod.RIGHT_FRONT).set(
      rf_optimized.angle.getDegrees(),
      rf_optimized.speedMetersPerSecond);
    getPod(Pod.LEFT_BACK).set(
      lb_optimized.angle.getDegrees(),
      lb_optimized.speedMetersPerSecond);
    getPod(Pod.RIGHT_BACK).set(
      rb_optimized.angle.getDegrees(),
      rb_optimized.speedMetersPerSecond);
  }

  public void driveFR(double x, double y, double theta) {
    ChassisSpeeds speeds = ChassisSpeeds.fromFieldRelativeSpeeds(
      x,
      y,
      theta,
      Rotation2d.fromDegrees(-m_gyro.getAngle()));
    drive(speeds);
  }

  public void zeroPodEncoders() {
    m_lf_pod.zeroEncoder();
    m_rf_pod.zeroEncoder();
    m_lb_pod.zeroEncoder();
    m_rb_pod.zeroEncoder();
  }

  public void zeroGyro() {
    m_gyro.zeroYaw();
  }

  public SwervePod getPod(Pod pod) {
    switch(pod) {
      case LEFT_FRONT:
        return m_lf_pod;
      case RIGHT_FRONT:
        return m_rf_pod;
      case LEFT_BACK:
        return m_lb_pod;
      case RIGHT_BACK:
        return m_rb_pod;
      default:
        throw new RuntimeException("Invalid pod?");
    }
  }

  @Override
  public void periodic() {}

  @Override
  public void simulationPeriodic() {}
}
